@@ -23,6 +23,7 @@ import "context"
 type Plugin interface {
 	Tag() string
 	Type() string
+	Shutdown() error
 }
 
 // Executable represents something that is executable.
@@ -53,18 +54,6 @@ type MatcherPlugin interface {
 	Matcher
 }
 
-// Service represents a background service.
-type Service interface {
-	// Shutdown and release resources.
-	Shutdown() error
-}
-
-// ServicePlugin represents a Plugin that is a Service.
-type ServicePlugin interface {
-	Plugin
-	Service
-}
-
 // ExecutableNodeWrapper wraps a Executable to a ExecutableChainNode.
 type ExecutableNodeWrapper struct {
 	Executable
