@@ -116,12 +116,11 @@ func newForwarder(bp *handler.BP, args *Args) (*forwardPlugin, error) {
 		})
 	}
 
+	f.bu = bundled_upstream.NewBundledUpstream(bu, bp.L())
+
 	if args.FastestIP {
 		f.fastIPHandler = fastip.NewFastestAddr()
-	} else {
-		f.bu = bundled_upstream.NewBundledUpstream(bu, bp.L())
 	}
-
 	return f, nil
 }
 
@@ -134,8 +133,25 @@ func (u *upstreamWrapper) Address() string {
 	return u.dnsproxyUpstream.Address()
 }
 
-func (u *upstreamWrapper) Exchange(_ context.Context, q *dns.Msg) (*dns.Msg, error) {
-	return u.dnsproxyUpstream.Exchange(q)
+func (u *upstreamWrapper) Exchange(ctx context.Context, q *dns.Msg) (*dns.Msg, error) {
+	type res struct {
+		r   *dns.Msg
+		err error
+	}
+	resChan := make(chan *res, 1)
+
+	go func() {
+		r := new(res)
+		r.r, r.err = u.dnsproxyUpstream.Exchange(q)
+		resChan <- r
+	}()
+
+	select {
+	case r := <-resChan:
+		return r.r, r.err
+	case <-ctx.Done():
+		return nil, ctx.Err()
+	}
 }
 
 func (u *upstreamWrapper) Trusted() bool {
@@ -158,8 +174,13 @@ func (f *forwardPlugin) Exec(ctx context.Context, qCtx *handler.Context, next ha
 func (f *forwardPlugin) exec(ctx context.Context, qCtx *handler.Context) error {
 	var r *dns.Msg
 	var err error
-	if f.fastIPHandler != nil {
-		r, _, err = f.fastIPHandler.ExchangeFastest(qCtx.Q().Copy(), f.upstreams)
+	q := qCtx.Q()
+	if f.fastIPHandler != nil && len(q.Question) == 1 {
+		// Only call ExchangeFastest if the query has one question.
+		// ExchangeFastest will use the first question name as host name.
+		// It won't check questions range. So it will it panic if len(q.Question) == 0.
+		// Ref: https://github.com/IrineSistiana/mosdns/issues/240
+		r, _, err = f.fastIPHandler.ExchangeFastest(q.Copy(), f.upstreams)
 	} else {
 		r, err = f.bu.ExchangeParallel(ctx, qCtx)
 	}
